package net.vpnsdk.vpn.util;

import android.util.Log;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class StringUtils {

    public static boolean isEmpty(String url) {
        if (null == url || "".equals(url))
            return true;
        for (int i = 0; i < url.length(); i++) {
            char c = url.charAt(i);
            if (c != ' ' && c != '\t' && c != '\n' && c != '\r') {
                return false;
            }
        }
        return true;
    }


    public static String Date2Str(Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
        return sdf.format(date);
    }

    public static boolean isURL(String url) {
        if (isEmpty(url)) {
            return false;
        }
        //not support parameters
        String regex = "^((https|http|ftp|rtsp|mms)?://)?"//http or https
                + "(([0-9]{1,3}\\.){3}[0-9]{1,3}" // 199.194.52.184
                + "|" //
                + "([0-9a-z_!~*'()-]+\\.)*" // www.
                + "([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\\." //baidu. or sina.
                + "[a-z]{2,6})" // .com or .museum
                + "(:[0-9]{1,4})?" // port
                + "((/?)|" // '/' isn't required if there is no file name
                + "(/[0-9a-z_!~*'().;?:@&=+$,%#-]+)+/?)$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(url.toLowerCase(Locale.getDefault()));
        try {
            return matcher.matches();
        } catch (Exception e) {
            return false;
        }
    }

    public static boolean isValidIP(String addr) {
        //For decimal IP,such as 192.168.0.1
        if (addr.length() < 7 || addr.length() > 15 || "".equals(addr)) {
            return false;
        }
        String rexp = "((?:(?:25[0-5]|2[0-4]\\d|((1\\d{2})|([1-9]?\\d)))\\.){3}(?:25[0-5]|2[0-4]\\d|((1\\d{2})|([1-9]?\\d))))";
        Pattern pat = Pattern.compile(rexp);
        Matcher mat = pat.matcher(addr);
        boolean ipAddress = mat.matches();
        return ipAddress;
    }

    public static String integerToStringIP(int ip) {
        return (ip & 0xFF) + "." +

                ((ip >> 8) & 0xFF) + "." +

                ((ip >> 16) & 0xFF) + "." +

                ((ip >> 24) & 0xFF);
    }

    public static Long StringToIntIP(String addr) {
        String[] addrArray = addr.split("\\.");

        long num = 0;

        for (int i = 0; i < addrArray.length; i++) {

            int power = 3 - i;

            num += ((Integer.parseInt(addrArray[i]) % 256 * Math.pow(256, power)));

        }

        return num;

    }
    /**
     * 用于扫描登录中使用的随机数
     * 格式是32位字母数字+时间戳
     * 获取扫码需要调用NativeLib中的函数getQRCode;
     * @return 生成的随机字符串
     */
    public  static String getRandomStr() {

        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        //获取长度为6的字符串
        for (int i = 0; i < 32; i++) {//32位随机字符
            //获取范围在3之内的索引值
            int number = random.nextInt(3);
            int result = 0;
            switch (number) {
                case 0:
                    //Math.random()*25+65成成65-90的int型的整型,强转小数只取整数部分
                    result = (int) (Math.random() * 25 + 65);  //对应A-Z 参考ASCII编码表
                    //将整型强转为char类型
                    sb.append((char) result);
                    break;
                case 1:
                    result = (int) (Math.random() * 25 + 97);   //对应a-z
                    sb.append((char) result);
                    break;
                case 2:
                    sb.append(new Random().nextInt(10));
                    break;
            }
        }
        //时间戳
        String timeStr = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        sb.append(timeStr);
        String str = sb.toString();
        //Log.i(Tag, "Random Str: " + str);
        return str;
    }
}
